import { ObservableArray } from '@nativescript/core';
import { toRaw, watch, isReactive, isRef } from 'nativescript-vue';
const baseExcludeCompareFields = { startingSide: null, menuOpened: null };
/**
 * Reactive synchronization between a reactive array with ObservableArray.
 *
 * @param arrayWatchTarge
 * @param options
 */
export function useSyncObservableArray(arrayRef, options = { addRemoveByField: '' }) {
    const { checkRemoved = true, checkAdded = true, checkUpdates = true, excludeCompareFields = undefined, addRemoveByField } = options;
    const excludeFields = {
        ...baseExcludeCompareFields,
        ...excludeCompareFields?.reduce((a, b) => {
            a[b] = null;
            return a;
        }, {}),
    };
    const observableArray = new ObservableArray(getClearArray(arrayRef));
    if (options?.watchUpdates && (isReactive(arrayRef) || isRef(arrayRef))) {
        watch(arrayRef, sync, { deep: true });
    }
    function sync(newArray) {
        //console.time("TIME_[useSyncObservableArray.sync]");
        const itemList = newArray ? getClearArray(newArray) : getClearArray(arrayRef);
        //console.log('Processing_[useSyncObservableArray.sync.itemList.length] ' + itemList.length);
        if (checkRemoved) {
            const indexRemoved = [];
            observableArray.forEach((itemObservable, index) => {
                const findItem = addRemoveByField ? itemList.find((item) => item[addRemoveByField] === itemObservable[addRemoveByField]) : itemList.find((item) => isEqualObject(item, itemObservable, excludeFields));
                if (!findItem) {
                    indexRemoved.push(index);
                }
            });
            indexRemoved.forEach((index) => observableArray.splice(index, 1));
        }
        if (checkAdded) {
            const indexAdd = [];
            itemList.forEach((item, index) => {
                const findItem = addRemoveByField ? observableArray.find((itemObservable) => itemObservable[addRemoveByField] === item[addRemoveByField]) : observableArray.find((itemObservable) => isEqualObject(itemObservable, item, excludeFields));
                if (!findItem) {
                    indexAdd.push(index);
                }
            });
            indexAdd.forEach((index) => observableArray.splice(index, 0, itemList[index]));
        }
        if (checkUpdates) {
            itemList.forEach((item, index) => {
                const itemObservable = observableArray.getItem(index);
                if (!isEqualObject(itemObservable, item, excludeFields)) {
                    observableArray.setItem(index, item);
                }
            });
        }
        //console.timeEnd("TIME_[useSyncObservableArray.sync]");
    }
    return {
        sync,
        observableArray,
    };
}
function getClearArray(array) {
    return cloneObject(extractArray(array));
}
function extractArray(array) {
    return isRef(array) ? array.value : array;
}
function isEqualObject(a, b, excludeFields) {
    const aObject = { ...a, ...excludeFields };
    const bObject = { ...b, ...excludeFields };
    return JSON.stringify(aObject) === JSON.stringify(bObject);
}
function cloneObject(object) {
    return JSON.parse(JSON.stringify(toRaw(object)));
}
//# sourceMappingURL=index.js.map